"""Validation helpers for switch_controller/managed_switch - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "switch-id",  # Managed-switch name.
    "sn",  # Managed-switch serial number.
    "fsw-wan1-peer",  # FortiSwitch WAN1 peer port.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "switch-id": "",
    "sn": "",
    "description": "",
    "switch-profile": "default",
    "access-profile": "default",
    "purdue-level": "3",
    "fsw-wan1-peer": "",
    "fsw-wan1-admin": "discovered",
    "poe-pre-standard-detection": "disable",
    "dhcp-server-access-list": "global",
    "poe-detection-type": 0,
    "max-poe-budget": 0,
    "directly-connected": 0,
    "version": 0,
    "max-allowed-trunk-members": 0,
    "pre-provisioned": 0,
    "l3-discovered": 0,
    "mgmt-mode": 0,
    "tunnel-discovered": 0,
    "tdr-supported": "",
    "dynamic-capability": "0x00000000000000000000000000000000",
    "switch-device-tag": "",
    "switch-dhcp_opt43_key": "",
    "mclag-igmp-snooping-aware": "enable",
    "dynamically-discovered": 0,
    "ptp-status": "disable",
    "ptp-profile": "default",
    "radius-nas-ip-override": "disable",
    "radius-nas-ip": "0.0.0.0",
    "route-offload": "disable",
    "route-offload-mclag": "disable",
    "type": "physical",
    "owner-vdom": "",
    "flow-identity": "00000000",
    "staged-image-version": "",
    "delayed-restart-trigger": 0,
    "firmware-provision": "disable",
    "firmware-provision-version": "",
    "firmware-provision-latest": "disable",
    "override-snmp-sysinfo": "disable",
    "override-snmp-trap-threshold": "disable",
    "override-snmp-community": "disable",
    "override-snmp-user": "disable",
    "qos-drop-policy": "taildrop",
    "qos-red-probability": 12,
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "switch-id": "string",  # Managed-switch name.
    "sn": "string",  # Managed-switch serial number.
    "description": "string",  # Description.
    "switch-profile": "string",  # FortiSwitch profile.
    "access-profile": "string",  # FortiSwitch access profile.
    "purdue-level": "option",  # Purdue Level of this FortiSwitch.
    "fsw-wan1-peer": "string",  # FortiSwitch WAN1 peer port.
    "fsw-wan1-admin": "option",  # FortiSwitch WAN1 admin status; enable to authorize the Forti
    "poe-pre-standard-detection": "option",  # Enable/disable PoE pre-standard detection.
    "dhcp-server-access-list": "option",  # DHCP snooping server access list.
    "poe-detection-type": "integer",  # PoE detection type for FortiSwitch.
    "max-poe-budget": "integer",  # Max PoE budget for FortiSwitch.
    "directly-connected": "integer",  # Directly connected FortiSwitch.
    "version": "integer",  # FortiSwitch version.
    "max-allowed-trunk-members": "integer",  # FortiSwitch maximum allowed trunk members.
    "pre-provisioned": "integer",  # Pre-provisioned managed switch.
    "l3-discovered": "integer",  # Layer 3 management discovered.
    "mgmt-mode": "integer",  # FortiLink management mode.
    "tunnel-discovered": "integer",  # SOCKS tunnel management discovered.
    "tdr-supported": "string",  # TDR supported.
    "dynamic-capability": "user",  # List of features this FortiSwitch supports (not configurable
    "switch-device-tag": "string",  # User definable label/tag.
    "switch-dhcp_opt43_key": "string",  # DHCP option43 key.
    "mclag-igmp-snooping-aware": "option",  # Enable/disable MCLAG IGMP-snooping awareness.
    "dynamically-discovered": "integer",  # Dynamically discovered FortiSwitch.
    "ptp-status": "option",  # Enable/disable PTP profile on this FortiSwitch.
    "ptp-profile": "string",  # PTP profile configuration.
    "radius-nas-ip-override": "option",  # Use locally defined NAS-IP.
    "radius-nas-ip": "ipv4-address",  # NAS-IP address.
    "route-offload": "option",  # Enable/disable route offload on this FortiSwitch.
    "route-offload-mclag": "option",  # Enable/disable route offload MCLAG on this FortiSwitch.
    "route-offload-router": "string",  # Configure route offload MCLAG IP address.
    "vlan": "string",  # Configure VLAN assignment priority.
    "type": "option",  # Indication of switch type, physical or virtual.
    "owner-vdom": "string",  # VDOM which owner of port belongs to.
    "flow-identity": "user",  # Flow-tracking netflow ipfix switch identity in hex format(00
    "staged-image-version": "string",  # Staged image version for FortiSwitch.
    "delayed-restart-trigger": "integer",  # Delayed restart triggered for this FortiSwitch.
    "firmware-provision": "option",  # Enable/disable provisioning of firmware to FortiSwitches on 
    "firmware-provision-version": "string",  # Firmware version to provision to this FortiSwitch on bootup 
    "firmware-provision-latest": "option",  # Enable/disable one-time automatic provisioning of the latest
    "ports": "string",  # Managed-switch port list.
    "ip-source-guard": "string",  # IP source guard.
    "stp-settings": "string",  # Configuration method to edit Spanning Tree Protocol (STP) se
    "stp-instance": "string",  # Configuration method to edit Spanning Tree Protocol (STP) in
    "override-snmp-sysinfo": "option",  # Enable/disable overriding the global SNMP system information
    "snmp-sysinfo": "string",  # Configuration method to edit Simple Network Management Proto
    "override-snmp-trap-threshold": "option",  # Enable/disable overriding the global SNMP trap threshold val
    "snmp-trap-threshold": "string",  # Configuration method to edit Simple Network Management Proto
    "override-snmp-community": "option",  # Enable/disable overriding the global SNMP communities.
    "snmp-community": "string",  # Configuration method to edit Simple Network Management Proto
    "override-snmp-user": "option",  # Enable/disable overriding the global SNMP users.
    "snmp-user": "string",  # Configuration method to edit Simple Network Management Proto
    "qos-drop-policy": "option",  # Set QoS drop-policy.
    "qos-red-probability": "integer",  # Set QoS RED/WRED drop probability.
    "switch-log": "string",  # Configuration method to edit FortiSwitch logging settings (l
    "remote-log": "string",  # Configure logging by FortiSwitch device to a remote syslog s
    "storm-control": "string",  # Configuration method to edit FortiSwitch storm control for m
    "mirror": "string",  # Configuration method to edit FortiSwitch packet mirror.
    "static-mac": "string",  # Configuration method to edit FortiSwitch Static and Sticky M
    "custom-command": "string",  # Configuration method to edit FortiSwitch commands to be push
    "dhcp-snooping-static-client": "string",  # Configure FortiSwitch DHCP snooping static clients.
    "igmp-snooping": "string",  # Configure FortiSwitch IGMP snooping global settings.
    "802-1X-settings": "string",  # Configuration method to edit FortiSwitch 802.1X global setti
    "router-vrf": "string",  # Configure VRF.
    "system-interface": "string",  # Configure system interface on FortiSwitch.
    "router-static": "string",  # Configure static routes.
    "system-dhcp-server": "string",  # Configure DHCP servers.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "switch-id": "Managed-switch name.",
    "sn": "Managed-switch serial number.",
    "description": "Description.",
    "switch-profile": "FortiSwitch profile.",
    "access-profile": "FortiSwitch access profile.",
    "purdue-level": "Purdue Level of this FortiSwitch.",
    "fsw-wan1-peer": "FortiSwitch WAN1 peer port.",
    "fsw-wan1-admin": "FortiSwitch WAN1 admin status; enable to authorize the FortiSwitch as a managed switch.",
    "poe-pre-standard-detection": "Enable/disable PoE pre-standard detection.",
    "dhcp-server-access-list": "DHCP snooping server access list.",
    "poe-detection-type": "PoE detection type for FortiSwitch.",
    "max-poe-budget": "Max PoE budget for FortiSwitch.",
    "directly-connected": "Directly connected FortiSwitch.",
    "version": "FortiSwitch version.",
    "max-allowed-trunk-members": "FortiSwitch maximum allowed trunk members.",
    "pre-provisioned": "Pre-provisioned managed switch.",
    "l3-discovered": "Layer 3 management discovered.",
    "mgmt-mode": "FortiLink management mode.",
    "tunnel-discovered": "SOCKS tunnel management discovered.",
    "tdr-supported": "TDR supported.",
    "dynamic-capability": "List of features this FortiSwitch supports (not configurable) that is sent to the FortiGate device for subsequent configuration initiated by the FortiGate device.",
    "switch-device-tag": "User definable label/tag.",
    "switch-dhcp_opt43_key": "DHCP option43 key.",
    "mclag-igmp-snooping-aware": "Enable/disable MCLAG IGMP-snooping awareness.",
    "dynamically-discovered": "Dynamically discovered FortiSwitch.",
    "ptp-status": "Enable/disable PTP profile on this FortiSwitch.",
    "ptp-profile": "PTP profile configuration.",
    "radius-nas-ip-override": "Use locally defined NAS-IP.",
    "radius-nas-ip": "NAS-IP address.",
    "route-offload": "Enable/disable route offload on this FortiSwitch.",
    "route-offload-mclag": "Enable/disable route offload MCLAG on this FortiSwitch.",
    "route-offload-router": "Configure route offload MCLAG IP address.",
    "vlan": "Configure VLAN assignment priority.",
    "type": "Indication of switch type, physical or virtual.",
    "owner-vdom": "VDOM which owner of port belongs to.",
    "flow-identity": "Flow-tracking netflow ipfix switch identity in hex format(00000000-FFFFFFFF default=0).",
    "staged-image-version": "Staged image version for FortiSwitch.",
    "delayed-restart-trigger": "Delayed restart triggered for this FortiSwitch.",
    "firmware-provision": "Enable/disable provisioning of firmware to FortiSwitches on join connection.",
    "firmware-provision-version": "Firmware version to provision to this FortiSwitch on bootup (major.minor.build, i.e. 6.2.1234).",
    "firmware-provision-latest": "Enable/disable one-time automatic provisioning of the latest firmware version.",
    "ports": "Managed-switch port list.",
    "ip-source-guard": "IP source guard.",
    "stp-settings": "Configuration method to edit Spanning Tree Protocol (STP) settings used to prevent bridge loops.",
    "stp-instance": "Configuration method to edit Spanning Tree Protocol (STP) instances.",
    "override-snmp-sysinfo": "Enable/disable overriding the global SNMP system information.",
    "snmp-sysinfo": "Configuration method to edit Simple Network Management Protocol (SNMP) system info.",
    "override-snmp-trap-threshold": "Enable/disable overriding the global SNMP trap threshold values.",
    "snmp-trap-threshold": "Configuration method to edit Simple Network Management Protocol (SNMP) trap threshold values.",
    "override-snmp-community": "Enable/disable overriding the global SNMP communities.",
    "snmp-community": "Configuration method to edit Simple Network Management Protocol (SNMP) communities.",
    "override-snmp-user": "Enable/disable overriding the global SNMP users.",
    "snmp-user": "Configuration method to edit Simple Network Management Protocol (SNMP) users.",
    "qos-drop-policy": "Set QoS drop-policy.",
    "qos-red-probability": "Set QoS RED/WRED drop probability.",
    "switch-log": "Configuration method to edit FortiSwitch logging settings (logs are transferred to and inserted into the FortiGate event log).",
    "remote-log": "Configure logging by FortiSwitch device to a remote syslog server.",
    "storm-control": "Configuration method to edit FortiSwitch storm control for measuring traffic activity using data rates to prevent traffic disruption.",
    "mirror": "Configuration method to edit FortiSwitch packet mirror.",
    "static-mac": "Configuration method to edit FortiSwitch Static and Sticky MAC.",
    "custom-command": "Configuration method to edit FortiSwitch commands to be pushed to this FortiSwitch device upon rebooting the FortiGate switch controller or the FortiSwitch.",
    "dhcp-snooping-static-client": "Configure FortiSwitch DHCP snooping static clients.",
    "igmp-snooping": "Configure FortiSwitch IGMP snooping global settings.",
    "802-1X-settings": "Configuration method to edit FortiSwitch 802.1X global settings.",
    "router-vrf": "Configure VRF.",
    "system-interface": "Configure system interface on FortiSwitch.",
    "router-static": "Configure static routes.",
    "system-dhcp-server": "Configure DHCP servers.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "switch-id": {"type": "string", "max_length": 35},
    "sn": {"type": "string", "max_length": 16},
    "description": {"type": "string", "max_length": 63},
    "switch-profile": {"type": "string", "max_length": 35},
    "access-profile": {"type": "string", "max_length": 31},
    "fsw-wan1-peer": {"type": "string", "max_length": 35},
    "poe-detection-type": {"type": "integer", "min": 0, "max": 255},
    "max-poe-budget": {"type": "integer", "min": 0, "max": 65535},
    "directly-connected": {"type": "integer", "min": 0, "max": 1},
    "version": {"type": "integer", "min": 0, "max": 255},
    "max-allowed-trunk-members": {"type": "integer", "min": 0, "max": 255},
    "pre-provisioned": {"type": "integer", "min": 0, "max": 255},
    "l3-discovered": {"type": "integer", "min": 0, "max": 1},
    "mgmt-mode": {"type": "integer", "min": 0, "max": 255},
    "tunnel-discovered": {"type": "integer", "min": 0, "max": 1},
    "tdr-supported": {"type": "string", "max_length": 31},
    "switch-device-tag": {"type": "string", "max_length": 32},
    "switch-dhcp_opt43_key": {"type": "string", "max_length": 63},
    "dynamically-discovered": {"type": "integer", "min": 0, "max": 1},
    "ptp-profile": {"type": "string", "max_length": 63},
    "owner-vdom": {"type": "string", "max_length": 31},
    "staged-image-version": {"type": "string", "max_length": 127},
    "delayed-restart-trigger": {"type": "integer", "min": 0, "max": 255},
    "firmware-provision-version": {"type": "string", "max_length": 35},
    "qos-red-probability": {"type": "integer", "min": 0, "max": 100},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "route-offload-router": {
        "vlan-name": {
            "type": "string",
            "help": "VLAN name.",
            "default": "",
            "max_length": 15,
        },
        "router-ip": {
            "type": "ipv4-address",
            "help": "Router IP address.",
            "required": True,
            "default": "0.0.0.0",
        },
    },
    "vlan": {
        "vlan-name": {
            "type": "string",
            "help": "VLAN name.",
            "default": "",
            "max_length": 15,
        },
        "assignment-priority": {
            "type": "integer",
            "help": "802.1x Radius (Tunnel-Private-Group-Id) VLANID assign-by-name priority. A smaller value has a higher priority.",
            "required": True,
            "default": 128,
            "min_value": 1,
            "max_value": 255,
        },
    },
    "ports": {
        "port-name": {
            "type": "string",
            "help": "Switch port name.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "port-owner": {
            "type": "string",
            "help": "Switch port name.",
            "default": "",
            "max_length": 15,
        },
        "switch-id": {
            "type": "string",
            "help": "Switch id.",
            "default": "",
            "max_length": 35,
        },
        "speed": {
            "type": "option",
            "help": "Switch port speed; default and available settings depend on hardware.",
            "default": "auto",
            "options": ["10half", "10full", "100half", "100full", "1000full", "10000full", "auto", "1000auto", "1000full-fiber", "40000full", "auto-module", "100FX-half", "100FX-full", "100000full", "2500auto", "2500full", "25000full", "50000full", "10000cr", "10000sr", "100000sr4", "100000cr4", "40000sr4", "40000cr4", "40000auto", "25000cr", "25000sr", "50000cr", "50000sr", "5000auto", "sgmii-auto"],
        },
        "status": {
            "type": "option",
            "help": "Switch port admin status: up or down.",
            "default": "up",
            "options": ["up", "down"],
        },
        "poe-status": {
            "type": "option",
            "help": "Enable/disable PoE status.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "ip-source-guard": {
            "type": "option",
            "help": "Enable/disable IP source guard.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "ptp-status": {
            "type": "option",
            "help": "Enable/disable PTP policy on this FortiSwitch port.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "ptp-policy": {
            "type": "string",
            "help": "PTP policy configuration.",
            "default": "default",
            "max_length": 63,
        },
        "aggregator-mode": {
            "type": "option",
            "help": "LACP member select mode.",
            "default": "bandwidth",
            "options": ["bandwidth", "count"],
        },
        "flapguard": {
            "type": "option",
            "help": "Enable/disable flap guard.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "flap-rate": {
            "type": "integer",
            "help": "Number of stage change events needed within flap-duration.",
            "default": 5,
            "min_value": 1,
            "max_value": 30,
        },
        "flap-duration": {
            "type": "integer",
            "help": "Period over which flap events are calculated (seconds).",
            "default": 30,
            "min_value": 5,
            "max_value": 300,
        },
        "flap-timeout": {
            "type": "integer",
            "help": "Flap guard disabling protection (min).",
            "default": 0,
            "min_value": 0,
            "max_value": 120,
        },
        "rpvst-port": {
            "type": "option",
            "help": "Enable/disable inter-operability with rapid PVST on this interface.",
            "default": "disabled",
            "options": ["disabled", "enabled"],
        },
        "poe-pre-standard-detection": {
            "type": "option",
            "help": "Enable/disable PoE pre-standard detection.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "port-number": {
            "type": "integer",
            "help": "Port number.",
            "default": 0,
            "min_value": 1,
            "max_value": 64,
        },
        "port-prefix-type": {
            "type": "integer",
            "help": "Port prefix type.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "fortilink-port": {
            "type": "integer",
            "help": "FortiLink uplink port.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "poe-capable": {
            "type": "integer",
            "help": "PoE capable.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "pd-capable": {
            "type": "integer",
            "help": "Powered device capable.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "stacking-port": {
            "type": "integer",
            "help": "Stacking port.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "p2p-port": {
            "type": "integer",
            "help": "General peer to peer tunnel port.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "mclag-icl-port": {
            "type": "integer",
            "help": "MCLAG-ICL port.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "authenticated-port": {
            "type": "integer",
            "help": "Peer to Peer Authenticated port.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "restricted-auth-port": {
            "type": "integer",
            "help": "Peer to Peer Restricted Authenticated port.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "encrypted-port": {
            "type": "integer",
            "help": "Peer to Peer Encrypted port.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "fiber-port": {
            "type": "integer",
            "help": "Fiber-port.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "media-type": {
            "type": "string",
            "help": "Media type.",
            "default": "",
            "max_length": 31,
        },
        "poe-standard": {
            "type": "string",
            "help": "PoE standard supported.",
            "default": "",
            "max_length": 63,
        },
        "poe-max-power": {
            "type": "string",
            "help": "PoE maximum power.",
            "default": "",
            "max_length": 35,
        },
        "poe-mode-bt-cabable": {
            "type": "integer",
            "help": "PoE mode IEEE 802.3BT capable.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "poe-port-mode": {
            "type": "option",
            "help": "Configure PoE port mode.",
            "default": "ieee802-3at",
            "options": ["ieee802-3af", "ieee802-3at", "ieee802-3bt"],
        },
        "poe-port-priority": {
            "type": "option",
            "help": "Configure PoE port priority.",
            "default": "low-priority",
            "options": ["critical-priority", "high-priority", "low-priority", "medium-priority"],
        },
        "poe-port-power": {
            "type": "option",
            "help": "Configure PoE port power.",
            "default": "normal",
            "options": ["normal", "perpetual", "perpetual-fast"],
        },
        "flags": {
            "type": "integer",
            "help": "Port properties flags.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "isl-local-trunk-name": {
            "type": "string",
            "help": "ISL local trunk name.",
            "default": "",
            "max_length": 15,
        },
        "isl-peer-port-name": {
            "type": "string",
            "help": "ISL peer port name.",
            "default": "",
            "max_length": 15,
        },
        "isl-peer-device-name": {
            "type": "string",
            "help": "ISL peer device name.",
            "default": "",
            "max_length": 35,
        },
        "isl-peer-device-sn": {
            "type": "string",
            "help": "ISL peer device serial number.",
            "default": "",
            "max_length": 16,
        },
        "fgt-peer-port-name": {
            "type": "string",
            "help": "FGT peer port name.",
            "default": "",
            "max_length": 15,
        },
        "fgt-peer-device-name": {
            "type": "string",
            "help": "FGT peer device name.",
            "default": "",
            "max_length": 35,
        },
        "vlan": {
            "type": "string",
            "help": "Assign switch ports to a VLAN.",
            "default": "",
            "max_length": 15,
        },
        "allowed-vlans-all": {
            "type": "option",
            "help": "Enable/disable all defined vlans on this port.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "allowed-vlans": {
            "type": "string",
            "help": "Configure switch port tagged VLANs.",
        },
        "untagged-vlans": {
            "type": "string",
            "help": "Configure switch port untagged VLANs.",
        },
        "type": {
            "type": "option",
            "help": "Interface type: physical or trunk port.",
            "default": "physical",
            "options": ["physical", "trunk"],
        },
        "access-mode": {
            "type": "option",
            "help": "Access mode of the port.",
            "default": "static",
            "options": ["dynamic", "nac", "static"],
        },
        "matched-dpp-policy": {
            "type": "string",
            "help": "Matched child policy in the dynamic port policy.",
            "default": "",
            "max_length": 63,
        },
        "matched-dpp-intf-tags": {
            "type": "string",
            "help": "Matched interface tags in the dynamic port policy.",
            "default": "",
            "max_length": 63,
        },
        "acl-group": {
            "type": "string",
            "help": "ACL groups on this port.",
        },
        "fortiswitch-acls": {
            "type": "string",
            "help": "ACLs on this port.",
        },
        "dhcp-snooping": {
            "type": "option",
            "help": "Trusted or untrusted DHCP-snooping interface.",
            "default": "untrusted",
            "options": ["untrusted", "trusted"],
        },
        "dhcp-snoop-option82-trust": {
            "type": "option",
            "help": "Enable/disable allowance of DHCP with option-82 on untrusted interface.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "dhcp-snoop-option82-override": {
            "type": "string",
            "help": "Configure DHCP snooping option 82 override.",
        },
        "arp-inspection-trust": {
            "type": "option",
            "help": "Trusted or untrusted dynamic ARP inspection.",
            "default": "untrusted",
            "options": ["untrusted", "trusted"],
        },
        "igmp-snooping-flood-reports": {
            "type": "option",
            "help": "Enable/disable flooding of IGMP reports to this interface when igmp-snooping enabled.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "mcast-snooping-flood-traffic": {
            "type": "option",
            "help": "Enable/disable flooding of IGMP snooping traffic to this interface.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "stp-state": {
            "type": "option",
            "help": "Enable/disable Spanning Tree Protocol (STP) on this interface.",
            "default": "enabled",
            "options": ["enabled", "disabled"],
        },
        "stp-root-guard": {
            "type": "option",
            "help": "Enable/disable STP root guard on this interface.",
            "default": "disabled",
            "options": ["enabled", "disabled"],
        },
        "stp-bpdu-guard": {
            "type": "option",
            "help": "Enable/disable STP BPDU guard on this interface.",
            "default": "disabled",
            "options": ["enabled", "disabled"],
        },
        "stp-bpdu-guard-timeout": {
            "type": "integer",
            "help": "BPDU Guard disabling protection (0 - 120 min).",
            "default": 5,
            "min_value": 0,
            "max_value": 120,
        },
        "edge-port": {
            "type": "option",
            "help": "Enable/disable this interface as an edge port, bridging connections between workstations and/or computers.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "discard-mode": {
            "type": "option",
            "help": "Configure discard mode for port.",
            "default": "none",
            "options": ["none", "all-untagged", "all-tagged"],
        },
        "packet-sampler": {
            "type": "option",
            "help": "Enable/disable packet sampling on this interface.",
            "default": "disabled",
            "options": ["enabled", "disabled"],
        },
        "packet-sample-rate": {
            "type": "integer",
            "help": "Packet sampling rate (0 - 99999 p/sec).",
            "default": 512,
            "min_value": 0,
            "max_value": 99999,
        },
        "sflow-counter-interval": {
            "type": "integer",
            "help": "sFlow sampling counter polling interval in seconds (0 - 255).",
            "default": 0,
            "min_value": 0,
            "max_value": 255,
        },
        "sample-direction": {
            "type": "option",
            "help": "Packet sampling direction.",
            "default": "both",
            "options": ["tx", "rx", "both"],
        },
        "fec-capable": {
            "type": "integer",
            "help": "FEC capable.",
            "default": 0,
            "min_value": 0,
            "max_value": 1,
        },
        "fec-state": {
            "type": "option",
            "help": "State of forward error correction.",
            "default": "detect-by-module",
            "options": ["disabled", "cl74", "cl91", "detect-by-module"],
        },
        "flow-control": {
            "type": "option",
            "help": "Flow control direction.",
            "default": "disable",
            "options": ["disable", "tx", "rx", "both"],
        },
        "pause-meter": {
            "type": "integer",
            "help": "Configure ingress pause metering rate, in kbps (default = 0, disabled).",
            "default": 0,
            "min_value": 128,
            "max_value": 2147483647,
        },
        "pause-meter-resume": {
            "type": "option",
            "help": "Resume threshold for resuming traffic on ingress port.",
            "default": "50%",
            "options": ["75%", "50%", "25%"],
        },
        "loop-guard": {
            "type": "option",
            "help": "Enable/disable loop-guard on this interface, an STP optimization used to prevent network loops.",
            "default": "disabled",
            "options": ["enabled", "disabled"],
        },
        "loop-guard-timeout": {
            "type": "integer",
            "help": "Loop-guard timeout (0 - 120 min, default = 45).",
            "default": 45,
            "min_value": 0,
            "max_value": 120,
        },
        "port-policy": {
            "type": "string",
            "help": "Switch controller dynamic port policy from available options.",
            "default": "",
            "max_length": 63,
        },
        "qos-policy": {
            "type": "string",
            "help": "Switch controller QoS policy from available options.",
            "default": "default",
            "max_length": 63,
        },
        "storm-control-policy": {
            "type": "string",
            "help": "Switch controller storm control policy from available options.",
            "default": "default",
            "max_length": 63,
        },
        "port-security-policy": {
            "type": "string",
            "help": "Switch controller authentication policy to apply to this managed switch from available options.",
            "default": "",
            "max_length": 31,
        },
        "export-to-pool": {
            "type": "string",
            "help": "Switch controller export port to pool-list.",
            "default": "",
            "max_length": 35,
        },
        "interface-tags": {
            "type": "string",
            "help": "Tag(s) associated with the interface for various features including virtual port pool, dynamic port policy.",
        },
        "learning-limit": {
            "type": "integer",
            "help": "Limit the number of dynamic MAC addresses on this Port (1 - 128, 0 = no limit, default).",
            "default": 0,
            "min_value": 0,
            "max_value": 128,
        },
        "sticky-mac": {
            "type": "option",
            "help": "Enable or disable sticky-mac on the interface.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "lldp-status": {
            "type": "option",
            "help": "LLDP transmit and receive status.",
            "default": "tx-rx",
            "options": ["disable", "rx-only", "tx-only", "tx-rx"],
        },
        "lldp-profile": {
            "type": "string",
            "help": "LLDP port TLV profile.",
            "default": "default-auto-isl",
            "max_length": 63,
        },
        "export-to": {
            "type": "string",
            "help": "Export managed-switch port to a tenant VDOM.",
            "default": "",
            "max_length": 31,
        },
        "mac-addr": {
            "type": "mac-address",
            "help": "Port/Trunk MAC.",
            "default": "00:00:00:00:00:00",
        },
        "allow-arp-monitor": {
            "type": "option",
            "help": "Enable/Disable allow ARP monitor.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "qnq": {
            "type": "string",
            "help": "802.1AD VLANs in the VDom.",
            "default": "",
            "max_length": 15,
        },
        "log-mac-event": {
            "type": "option",
            "help": "Enable/disable logging for dynamic MAC address events.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "port-selection-criteria": {
            "type": "option",
            "help": "Algorithm for aggregate port selection.",
            "default": "src-dst-ip",
            "options": ["src-mac", "dst-mac", "src-dst-mac", "src-ip", "dst-ip", "src-dst-ip"],
        },
        "description": {
            "type": "string",
            "help": "Description for port.",
            "default": "",
            "max_length": 63,
        },
        "lacp-speed": {
            "type": "option",
            "help": "End Link Aggregation Control Protocol (LACP) messages every 30 seconds (slow) or every second (fast).",
            "default": "slow",
            "options": ["slow", "fast"],
        },
        "mode": {
            "type": "option",
            "help": "LACP mode: ignore and do not send control messages, or negotiate 802.3ad aggregation passively or actively.",
            "default": "static",
            "options": ["static", "lacp-passive", "lacp-active"],
        },
        "bundle": {
            "type": "option",
            "help": "Enable/disable Link Aggregation Group (LAG) bundling for non-FortiLink interfaces.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "member-withdrawal-behavior": {
            "type": "option",
            "help": "Port behavior after it withdraws because of loss of control packets.",
            "default": "block",
            "options": ["forward", "block"],
        },
        "mclag": {
            "type": "option",
            "help": "Enable/disable multi-chassis link aggregation (MCLAG).",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "min-bundle": {
            "type": "integer",
            "help": "Minimum size of LAG bundle (1 - 24, default = 1).",
            "default": 1,
            "min_value": 1,
            "max_value": 24,
        },
        "max-bundle": {
            "type": "integer",
            "help": "Maximum size of LAG bundle (1 - 24, default = 24).",
            "default": 24,
            "min_value": 1,
            "max_value": 24,
        },
        "members": {
            "type": "string",
            "help": "Aggregated LAG bundle interfaces.",
        },
        "fallback-port": {
            "type": "string",
            "help": "LACP fallback port.",
            "default": "",
            "max_length": 79,
        },
    },
    "ip-source-guard": {
        "port": {
            "type": "string",
            "help": "Ingress interface to which source guard is bound.",
            "default": "",
            "max_length": 15,
        },
        "description": {
            "type": "string",
            "help": "Description.",
            "default": "",
            "max_length": 63,
        },
        "binding-entry": {
            "type": "string",
            "help": "IP and MAC address configuration.",
            "required": True,
        },
    },
    "stp-settings": {
        "local-override": {
            "type": "option",
            "help": "Enable to configure local STP settings that override global STP settings.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "name": {
            "type": "string",
            "help": "Name of local STP settings configuration.",
            "default": "",
            "max_length": 31,
        },
        "revision": {
            "type": "integer",
            "help": "STP revision number (0 - 65535).",
            "default": 0,
            "min_value": 0,
            "max_value": 65535,
        },
        "hello-time": {
            "type": "integer",
            "help": "Period of time between successive STP frame Bridge Protocol Data Units (BPDUs) sent on a port (1 - 10 sec, default = 2).",
            "default": 2,
            "min_value": 1,
            "max_value": 10,
        },
        "forward-time": {
            "type": "integer",
            "help": "Period of time a port is in listening and learning state (4 - 30 sec, default = 15).",
            "default": 15,
            "min_value": 4,
            "max_value": 30,
        },
        "max-age": {
            "type": "integer",
            "help": "Maximum time before a bridge port saves its configuration BPDU information (6 - 40 sec, default = 20).",
            "default": 20,
            "min_value": 6,
            "max_value": 40,
        },
        "max-hops": {
            "type": "integer",
            "help": "Maximum number of hops between the root bridge and the furthest bridge (1- 40, default = 20).",
            "default": 20,
            "min_value": 1,
            "max_value": 40,
        },
        "pending-timer": {
            "type": "integer",
            "help": "Pending time (1 - 15 sec, default = 4).",
            "default": 4,
            "min_value": 1,
            "max_value": 15,
        },
    },
    "stp-instance": {
        "id": {
            "type": "string",
            "help": "Instance ID.",
            "default": "",
            "max_length": 2,
        },
        "priority": {
            "type": "option",
            "help": "Priority.",
            "default": "32768",
            "options": ["0", "4096", "8192", "12288", "16384", "20480", "24576", "28672", "32768", "36864", "40960", "45056", "49152", "53248", "57344", "61440"],
        },
    },
    "snmp-sysinfo": {
        "status": {
            "type": "option",
            "help": "Enable/disable SNMP.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "engine-id": {
            "type": "string",
            "help": "Local SNMP engine ID string (max 24 char).",
            "default": "",
            "max_length": 24,
        },
        "description": {
            "type": "string",
            "help": "System description.",
            "default": "",
            "max_length": 35,
        },
        "contact-info": {
            "type": "string",
            "help": "Contact information.",
            "default": "",
            "max_length": 35,
        },
        "location": {
            "type": "string",
            "help": "System location.",
            "default": "",
            "max_length": 35,
        },
    },
    "snmp-trap-threshold": {
        "trap-high-cpu-threshold": {
            "type": "integer",
            "help": "CPU usage when trap is sent.",
            "default": 80,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "trap-low-memory-threshold": {
            "type": "integer",
            "help": "Memory usage when trap is sent.",
            "default": 80,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "trap-log-full-threshold": {
            "type": "integer",
            "help": "Log disk usage when trap is sent.",
            "default": 90,
            "min_value": 0,
            "max_value": 4294967295,
        },
    },
    "snmp-community": {
        "id": {
            "type": "integer",
            "help": "SNMP community ID.",
            "required": True,
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "name": {
            "type": "string",
            "help": "SNMP community name.",
            "required": True,
            "default": "",
            "max_length": 35,
        },
        "status": {
            "type": "option",
            "help": "Enable/disable this SNMP community.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "hosts": {
            "type": "string",
            "help": "Configure IPv4 SNMP managers (hosts).",
        },
        "query-v1-status": {
            "type": "option",
            "help": "Enable/disable SNMP v1 queries.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "query-v1-port": {
            "type": "integer",
            "help": "SNMP v1 query port (default = 161).",
            "default": 161,
            "min_value": 0,
            "max_value": 65535,
        },
        "query-v2c-status": {
            "type": "option",
            "help": "Enable/disable SNMP v2c queries.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "query-v2c-port": {
            "type": "integer",
            "help": "SNMP v2c query port (default = 161).",
            "default": 161,
            "min_value": 0,
            "max_value": 65535,
        },
        "trap-v1-status": {
            "type": "option",
            "help": "Enable/disable SNMP v1 traps.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "trap-v1-lport": {
            "type": "integer",
            "help": "SNMP v2c trap local port (default = 162).",
            "default": 162,
            "min_value": 0,
            "max_value": 65535,
        },
        "trap-v1-rport": {
            "type": "integer",
            "help": "SNMP v2c trap remote port (default = 162).",
            "default": 162,
            "min_value": 0,
            "max_value": 65535,
        },
        "trap-v2c-status": {
            "type": "option",
            "help": "Enable/disable SNMP v2c traps.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "trap-v2c-lport": {
            "type": "integer",
            "help": "SNMP v2c trap local port (default = 162).",
            "default": 162,
            "min_value": 0,
            "max_value": 65535,
        },
        "trap-v2c-rport": {
            "type": "integer",
            "help": "SNMP v2c trap remote port (default = 162).",
            "default": 162,
            "min_value": 0,
            "max_value": 65535,
        },
        "events": {
            "type": "option",
            "help": "SNMP notifications (traps) to send.",
            "default": "cpu-high mem-low log-full intf-ip ent-conf-change l2mac",
            "options": ["cpu-high", "mem-low", "log-full", "intf-ip", "ent-conf-change", "l2mac"],
        },
    },
    "snmp-user": {
        "name": {
            "type": "string",
            "help": "SNMP user name.",
            "default": "",
            "max_length": 32,
        },
        "queries": {
            "type": "option",
            "help": "Enable/disable SNMP queries for this user.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "query-port": {
            "type": "integer",
            "help": "SNMPv3 query port (default = 161).",
            "default": 161,
            "min_value": 0,
            "max_value": 65535,
        },
        "security-level": {
            "type": "option",
            "help": "Security level for message authentication and encryption.",
            "default": "no-auth-no-priv",
            "options": ["no-auth-no-priv", "auth-no-priv", "auth-priv"],
        },
        "auth-proto": {
            "type": "option",
            "help": "Authentication protocol.",
            "default": "sha256",
            "options": ["md5", "sha1", "sha224", "sha256", "sha384", "sha512"],
        },
        "auth-pwd": {
            "type": "password",
            "help": "Password for authentication protocol.",
            "required": True,
            "max_length": 128,
        },
        "priv-proto": {
            "type": "option",
            "help": "Privacy (encryption) protocol.",
            "default": "aes128",
            "options": ["aes128", "aes192", "aes192c", "aes256", "aes256c", "des"],
        },
        "priv-pwd": {
            "type": "password",
            "help": "Password for privacy (encryption) protocol.",
            "required": True,
            "max_length": 128,
        },
    },
    "switch-log": {
        "local-override": {
            "type": "option",
            "help": "Enable to configure local logging settings that override global logging settings.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "status": {
            "type": "option",
            "help": "Enable/disable adding FortiSwitch logs to the FortiGate event log.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "severity": {
            "type": "option",
            "help": "Severity of FortiSwitch logs that are added to the FortiGate event log.",
            "default": "notification",
            "options": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"],
        },
    },
    "remote-log": {
        "name": {
            "type": "string",
            "help": "Remote log name.",
            "required": True,
            "default": "",
            "max_length": 35,
        },
        "status": {
            "type": "option",
            "help": "Enable/disable logging by FortiSwitch device to a remote syslog server.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "server": {
            "type": "string",
            "help": "IPv4 address of the remote syslog server.",
            "required": True,
            "default": "",
            "max_length": 63,
        },
        "port": {
            "type": "integer",
            "help": "Remote syslog server listening port.",
            "default": 514,
            "min_value": 0,
            "max_value": 65535,
        },
        "severity": {
            "type": "option",
            "help": "Severity of logs to be transferred to remote log server.",
            "default": "information",
            "options": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"],
        },
        "csv": {
            "type": "option",
            "help": "Enable/disable comma-separated value (CSV) strings.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "facility": {
            "type": "option",
            "help": "Facility to log to remote syslog server.",
            "default": "local7",
            "options": ["kernel", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv", "ftp", "ntp", "audit", "alert", "clock", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"],
        },
    },
    "storm-control": {
        "local-override": {
            "type": "option",
            "help": "Enable to override global FortiSwitch storm control settings for this FortiSwitch.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "rate": {
            "type": "integer",
            "help": "Rate in packets per second at which storm control drops excess traffic(0-10000000, default=500, drop-all=0).",
            "default": 500,
            "min_value": 0,
            "max_value": 10000000,
        },
        "burst-size-level": {
            "type": "integer",
            "help": "Increase level to handle bursty traffic (0 - 4, default = 0).",
            "default": 0,
            "min_value": 0,
            "max_value": 4,
        },
        "unknown-unicast": {
            "type": "option",
            "help": "Enable/disable storm control to drop unknown unicast traffic.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "unknown-multicast": {
            "type": "option",
            "help": "Enable/disable storm control to drop unknown multicast traffic.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "broadcast": {
            "type": "option",
            "help": "Enable/disable storm control to drop broadcast traffic.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
    },
    "mirror": {
        "name": {
            "type": "string",
            "help": "Mirror name.",
            "default": "",
            "max_length": 63,
        },
        "status": {
            "type": "option",
            "help": "Active/inactive mirror configuration.",
            "default": "inactive",
            "options": ["active", "inactive"],
        },
        "switching-packet": {
            "type": "option",
            "help": "Enable/disable switching functionality when mirroring.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "dst": {
            "type": "string",
            "help": "Destination port.",
            "default": "",
            "max_length": 63,
        },
        "src-ingress": {
            "type": "string",
            "help": "Source ingress interfaces.",
        },
        "src-egress": {
            "type": "string",
            "help": "Source egress interfaces.",
        },
    },
    "static-mac": {
        "id": {
            "type": "integer",
            "help": "ID.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "type": {
            "type": "option",
            "help": "Type.",
            "default": "static",
            "options": ["static", "sticky"],
        },
        "vlan": {
            "type": "string",
            "help": "Vlan.",
            "default": "",
            "max_length": 15,
        },
        "mac": {
            "type": "mac-address",
            "help": "MAC address.",
            "default": "00:00:00:00:00:00",
        },
        "interface": {
            "type": "string",
            "help": "Interface name.",
            "default": "",
            "max_length": 35,
        },
        "description": {
            "type": "string",
            "help": "Description.",
            "default": "",
            "max_length": 63,
        },
    },
    "custom-command": {
        "command-entry": {
            "type": "string",
            "help": "List of FortiSwitch commands.",
            "default": "",
            "max_length": 35,
        },
        "command-name": {
            "type": "string",
            "help": "Names of commands to be pushed to this FortiSwitch device, as configured under config switch-controller custom-command.",
            "required": True,
            "default": "",
            "max_length": 35,
        },
    },
    "dhcp-snooping-static-client": {
        "name": {
            "type": "string",
            "help": "Client name.",
            "default": "",
            "max_length": 35,
        },
        "vlan": {
            "type": "string",
            "help": "VLAN name.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "ip": {
            "type": "ipv4-address",
            "help": "Client static IP address.",
            "required": True,
            "default": "0.0.0.0",
        },
        "mac": {
            "type": "mac-address",
            "help": "Client MAC address.",
            "required": True,
            "default": "00:00:00:00:00:00",
        },
        "port": {
            "type": "string",
            "help": "Interface name.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
    },
    "igmp-snooping": {
        "local-override": {
            "type": "option",
            "help": "Enable/disable overriding the global IGMP snooping configuration.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "aging-time": {
            "type": "integer",
            "help": "Maximum time to retain a multicast snooping entry for which no packets have been seen (15 - 3600 sec, default = 300).",
            "default": 300,
            "min_value": 15,
            "max_value": 3600,
        },
        "flood-unknown-multicast": {
            "type": "option",
            "help": "Enable/disable unknown multicast flooding.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "vlans": {
            "type": "string",
            "help": "Configure IGMP snooping VLAN.",
        },
    },
    "802-1X-settings": {
        "local-override": {
            "type": "option",
            "help": "Enable to override global 802.1X settings on individual FortiSwitches.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "link-down-auth": {
            "type": "option",
            "help": "Authentication state to set if a link is down.",
            "default": "set-unauth",
            "options": ["set-unauth", "no-action"],
        },
        "reauth-period": {
            "type": "integer",
            "help": "Reauthentication time interval (1 - 1440 min, default = 60, 0 = disable).",
            "default": 60,
            "min_value": 0,
            "max_value": 1440,
        },
        "max-reauth-attempt": {
            "type": "integer",
            "help": "Maximum number of authentication attempts (0 - 15, default = 3).",
            "default": 3,
            "min_value": 0,
            "max_value": 15,
        },
        "tx-period": {
            "type": "integer",
            "help": "802.1X Tx period (seconds, default=30).",
            "default": 30,
            "min_value": 12,
            "max_value": 60,
        },
        "mab-reauth": {
            "type": "option",
            "help": "Enable or disable MAB reauthentication settings.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "mac-username-delimiter": {
            "type": "option",
            "help": "MAC authentication username delimiter (default = hyphen).",
            "default": "hyphen",
            "options": ["colon", "hyphen", "none", "single-hyphen"],
        },
        "mac-password-delimiter": {
            "type": "option",
            "help": "MAC authentication password delimiter (default = hyphen).",
            "default": "hyphen",
            "options": ["colon", "hyphen", "none", "single-hyphen"],
        },
        "mac-calling-station-delimiter": {
            "type": "option",
            "help": "MAC calling station delimiter (default = hyphen).",
            "default": "hyphen",
            "options": ["colon", "hyphen", "none", "single-hyphen"],
        },
        "mac-called-station-delimiter": {
            "type": "option",
            "help": "MAC called station delimiter (default = hyphen).",
            "default": "hyphen",
            "options": ["colon", "hyphen", "none", "single-hyphen"],
        },
        "mac-case": {
            "type": "option",
            "help": "MAC case (default = lowercase).",
            "default": "lowercase",
            "options": ["lowercase", "uppercase"],
        },
    },
    "router-vrf": {
        "name": {
            "type": "string",
            "help": "VRF entry name.",
            "required": True,
            "default": "",
            "max_length": 35,
        },
        "switch-id": {
            "type": "string",
            "help": "Switch ID.",
            "default": "",
            "max_length": 35,
        },
        "vrfid": {
            "type": "integer",
            "help": "VRF ID.",
            "required": True,
            "default": 0,
            "min_value": 0,
            "max_value": 1023,
        },
    },
    "system-interface": {
        "name": {
            "type": "string",
            "help": "Interface name.",
            "default": "",
            "max_length": 15,
        },
        "switch-id": {
            "type": "string",
            "help": "Switch ID.",
            "default": "",
            "max_length": 35,
        },
        "mode": {
            "type": "option",
            "help": "Interface addressing mode.",
            "default": "static",
            "options": ["static", "dhcp"],
        },
        "ip": {
            "type": "ipv4-classnet-host",
            "help": "IP and mask for this interface.",
            "required": True,
            "default": "0.0.0.0 0.0.0.0",
        },
        "status": {
            "type": "option",
            "help": "Enable/disable interface status.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "allowaccess": {
            "type": "option",
            "help": "Permitted types of management access to this interface.",
            "default": "",
            "options": ["ping", "https", "http", "ssh", "snmp", "telnet", "radius-acct"],
        },
        "vlan": {
            "type": "string",
            "help": "VLAN name.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "type": {
            "type": "option",
            "help": "Interface type.",
            "default": "vlan",
            "options": ["vlan", "physical"],
        },
        "interface": {
            "type": "string",
            "help": "Interface name.",
            "required": True,
            "default": "",
            "max_length": 63,
        },
        "vrf": {
            "type": "string",
            "help": "VRF for this route.",
            "default": "",
            "max_length": 63,
        },
    },
    "router-static": {
        "id": {
            "type": "integer",
            "help": "Entry sequence number.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "switch-id": {
            "type": "string",
            "help": "Switch ID.",
            "default": "",
            "max_length": 35,
        },
        "blackhole": {
            "type": "option",
            "help": "Enable/disable blackhole on this route.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "comment": {
            "type": "string",
            "help": "Comment.",
            "default": "",
            "max_length": 63,
        },
        "device": {
            "type": "string",
            "help": "Gateway out interface.",
            "default": "",
            "max_length": 35,
        },
        "distance": {
            "type": "integer",
            "help": "Administrative distance for the route (1 - 255, default = 10).",
            "default": 10,
            "min_value": 1,
            "max_value": 255,
        },
        "dst": {
            "type": "ipv4-classnet",
            "help": "Destination ip and mask for this route.",
            "required": True,
            "default": "0.0.0.0 0.0.0.0",
        },
        "dynamic-gateway": {
            "type": "option",
            "help": "Enable/disable dynamic gateway.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "gateway": {
            "type": "ipv4-address",
            "help": "Gateway ip for this route.",
            "required": True,
            "default": "0.0.0.0",
        },
        "status": {
            "type": "option",
            "help": "Enable/disable route status.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "vrf": {
            "type": "string",
            "help": "VRF for this route.",
            "default": "",
            "max_length": 35,
        },
    },
    "system-dhcp-server": {
        "id": {
            "type": "integer",
            "help": "ID.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "switch-id": {
            "type": "string",
            "help": "Switch ID.",
            "default": "",
            "max_length": 35,
        },
        "status": {
            "type": "option",
            "help": "Enable/disable this DHCP configuration.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "lease-time": {
            "type": "integer",
            "help": "Lease time in seconds, 0 means unlimited.",
            "default": 604800,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "dns-service": {
            "type": "option",
            "help": "Options for assigning DNS servers to DHCP clients.",
            "default": "specify",
            "options": ["local", "default", "specify"],
        },
        "dns-server1": {
            "type": "ipv4-address",
            "help": "DNS server 1.",
            "default": "0.0.0.0",
        },
        "dns-server2": {
            "type": "ipv4-address",
            "help": "DNS server 2.",
            "default": "0.0.0.0",
        },
        "dns-server3": {
            "type": "ipv4-address",
            "help": "DNS server 3.",
            "default": "0.0.0.0",
        },
        "ntp-service": {
            "type": "option",
            "help": "Options for assigning Network Time Protocol (NTP) servers to DHCP clients.",
            "default": "specify",
            "options": ["local", "default", "specify"],
        },
        "ntp-server1": {
            "type": "ipv4-address",
            "help": "NTP server 1.",
            "default": "0.0.0.0",
        },
        "ntp-server2": {
            "type": "ipv4-address",
            "help": "NTP server 2.",
            "default": "0.0.0.0",
        },
        "ntp-server3": {
            "type": "ipv4-address",
            "help": "NTP server 3.",
            "default": "0.0.0.0",
        },
        "default-gateway": {
            "type": "ipv4-address",
            "help": "Default gateway IP address assigned by the DHCP server.",
            "default": "0.0.0.0",
        },
        "netmask": {
            "type": "ipv4-netmask",
            "help": "Netmask assigned by the DHCP server.",
            "required": True,
            "default": "0.0.0.0",
        },
        "interface": {
            "type": "string",
            "help": "DHCP server can assign IP configurations to clients connected to this interface.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "ip-range": {
            "type": "string",
            "help": "DHCP IP range configuration.",
        },
        "options": {
            "type": "string",
            "help": "DHCP options.",
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_PURDUE_LEVEL = [
    "1",
    "1.5",
    "2",
    "2.5",
    "3",
    "3.5",
    "4",
    "5",
    "5.5",
]
VALID_BODY_FSW_WAN1_ADMIN = [
    "discovered",
    "disable",
    "enable",
]
VALID_BODY_POE_PRE_STANDARD_DETECTION = [
    "enable",
    "disable",
]
VALID_BODY_DHCP_SERVER_ACCESS_LIST = [
    "global",
    "enable",
    "disable",
]
VALID_BODY_MCLAG_IGMP_SNOOPING_AWARE = [
    "enable",
    "disable",
]
VALID_BODY_PTP_STATUS = [
    "disable",
    "enable",
]
VALID_BODY_RADIUS_NAS_IP_OVERRIDE = [
    "disable",
    "enable",
]
VALID_BODY_ROUTE_OFFLOAD = [
    "disable",
    "enable",
]
VALID_BODY_ROUTE_OFFLOAD_MCLAG = [
    "disable",
    "enable",
]
VALID_BODY_TYPE = [
    "virtual",
    "physical",
]
VALID_BODY_FIRMWARE_PROVISION = [
    "enable",
    "disable",
]
VALID_BODY_FIRMWARE_PROVISION_LATEST = [
    "disable",
    "once",
]
VALID_BODY_OVERRIDE_SNMP_SYSINFO = [
    "disable",
    "enable",
]
VALID_BODY_OVERRIDE_SNMP_TRAP_THRESHOLD = [
    "enable",
    "disable",
]
VALID_BODY_OVERRIDE_SNMP_COMMUNITY = [
    "enable",
    "disable",
]
VALID_BODY_OVERRIDE_SNMP_USER = [
    "enable",
    "disable",
]
VALID_BODY_QOS_DROP_POLICY = [
    "taildrop",
    "random-early-detection",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_switch_controller_managed_switch_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for switch_controller/managed_switch."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_switch_controller_managed_switch_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new switch_controller/managed_switch object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "purdue-level" in payload:
        is_valid, error = _validate_enum_field(
            "purdue-level",
            payload["purdue-level"],
            VALID_BODY_PURDUE_LEVEL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "fsw-wan1-admin" in payload:
        is_valid, error = _validate_enum_field(
            "fsw-wan1-admin",
            payload["fsw-wan1-admin"],
            VALID_BODY_FSW_WAN1_ADMIN,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "poe-pre-standard-detection" in payload:
        is_valid, error = _validate_enum_field(
            "poe-pre-standard-detection",
            payload["poe-pre-standard-detection"],
            VALID_BODY_POE_PRE_STANDARD_DETECTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "dhcp-server-access-list" in payload:
        is_valid, error = _validate_enum_field(
            "dhcp-server-access-list",
            payload["dhcp-server-access-list"],
            VALID_BODY_DHCP_SERVER_ACCESS_LIST,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "mclag-igmp-snooping-aware" in payload:
        is_valid, error = _validate_enum_field(
            "mclag-igmp-snooping-aware",
            payload["mclag-igmp-snooping-aware"],
            VALID_BODY_MCLAG_IGMP_SNOOPING_AWARE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ptp-status" in payload:
        is_valid, error = _validate_enum_field(
            "ptp-status",
            payload["ptp-status"],
            VALID_BODY_PTP_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "radius-nas-ip-override" in payload:
        is_valid, error = _validate_enum_field(
            "radius-nas-ip-override",
            payload["radius-nas-ip-override"],
            VALID_BODY_RADIUS_NAS_IP_OVERRIDE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "route-offload" in payload:
        is_valid, error = _validate_enum_field(
            "route-offload",
            payload["route-offload"],
            VALID_BODY_ROUTE_OFFLOAD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "route-offload-mclag" in payload:
        is_valid, error = _validate_enum_field(
            "route-offload-mclag",
            payload["route-offload-mclag"],
            VALID_BODY_ROUTE_OFFLOAD_MCLAG,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "firmware-provision" in payload:
        is_valid, error = _validate_enum_field(
            "firmware-provision",
            payload["firmware-provision"],
            VALID_BODY_FIRMWARE_PROVISION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "firmware-provision-latest" in payload:
        is_valid, error = _validate_enum_field(
            "firmware-provision-latest",
            payload["firmware-provision-latest"],
            VALID_BODY_FIRMWARE_PROVISION_LATEST,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "override-snmp-sysinfo" in payload:
        is_valid, error = _validate_enum_field(
            "override-snmp-sysinfo",
            payload["override-snmp-sysinfo"],
            VALID_BODY_OVERRIDE_SNMP_SYSINFO,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "override-snmp-trap-threshold" in payload:
        is_valid, error = _validate_enum_field(
            "override-snmp-trap-threshold",
            payload["override-snmp-trap-threshold"],
            VALID_BODY_OVERRIDE_SNMP_TRAP_THRESHOLD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "override-snmp-community" in payload:
        is_valid, error = _validate_enum_field(
            "override-snmp-community",
            payload["override-snmp-community"],
            VALID_BODY_OVERRIDE_SNMP_COMMUNITY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "override-snmp-user" in payload:
        is_valid, error = _validate_enum_field(
            "override-snmp-user",
            payload["override-snmp-user"],
            VALID_BODY_OVERRIDE_SNMP_USER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "qos-drop-policy" in payload:
        is_valid, error = _validate_enum_field(
            "qos-drop-policy",
            payload["qos-drop-policy"],
            VALID_BODY_QOS_DROP_POLICY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_switch_controller_managed_switch_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update switch_controller/managed_switch."""
    # Validate enum values using central function
    if "purdue-level" in payload:
        is_valid, error = _validate_enum_field(
            "purdue-level",
            payload["purdue-level"],
            VALID_BODY_PURDUE_LEVEL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "fsw-wan1-admin" in payload:
        is_valid, error = _validate_enum_field(
            "fsw-wan1-admin",
            payload["fsw-wan1-admin"],
            VALID_BODY_FSW_WAN1_ADMIN,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "poe-pre-standard-detection" in payload:
        is_valid, error = _validate_enum_field(
            "poe-pre-standard-detection",
            payload["poe-pre-standard-detection"],
            VALID_BODY_POE_PRE_STANDARD_DETECTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "dhcp-server-access-list" in payload:
        is_valid, error = _validate_enum_field(
            "dhcp-server-access-list",
            payload["dhcp-server-access-list"],
            VALID_BODY_DHCP_SERVER_ACCESS_LIST,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "mclag-igmp-snooping-aware" in payload:
        is_valid, error = _validate_enum_field(
            "mclag-igmp-snooping-aware",
            payload["mclag-igmp-snooping-aware"],
            VALID_BODY_MCLAG_IGMP_SNOOPING_AWARE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ptp-status" in payload:
        is_valid, error = _validate_enum_field(
            "ptp-status",
            payload["ptp-status"],
            VALID_BODY_PTP_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "radius-nas-ip-override" in payload:
        is_valid, error = _validate_enum_field(
            "radius-nas-ip-override",
            payload["radius-nas-ip-override"],
            VALID_BODY_RADIUS_NAS_IP_OVERRIDE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "route-offload" in payload:
        is_valid, error = _validate_enum_field(
            "route-offload",
            payload["route-offload"],
            VALID_BODY_ROUTE_OFFLOAD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "route-offload-mclag" in payload:
        is_valid, error = _validate_enum_field(
            "route-offload-mclag",
            payload["route-offload-mclag"],
            VALID_BODY_ROUTE_OFFLOAD_MCLAG,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "firmware-provision" in payload:
        is_valid, error = _validate_enum_field(
            "firmware-provision",
            payload["firmware-provision"],
            VALID_BODY_FIRMWARE_PROVISION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "firmware-provision-latest" in payload:
        is_valid, error = _validate_enum_field(
            "firmware-provision-latest",
            payload["firmware-provision-latest"],
            VALID_BODY_FIRMWARE_PROVISION_LATEST,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "override-snmp-sysinfo" in payload:
        is_valid, error = _validate_enum_field(
            "override-snmp-sysinfo",
            payload["override-snmp-sysinfo"],
            VALID_BODY_OVERRIDE_SNMP_SYSINFO,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "override-snmp-trap-threshold" in payload:
        is_valid, error = _validate_enum_field(
            "override-snmp-trap-threshold",
            payload["override-snmp-trap-threshold"],
            VALID_BODY_OVERRIDE_SNMP_TRAP_THRESHOLD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "override-snmp-community" in payload:
        is_valid, error = _validate_enum_field(
            "override-snmp-community",
            payload["override-snmp-community"],
            VALID_BODY_OVERRIDE_SNMP_COMMUNITY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "override-snmp-user" in payload:
        is_valid, error = _validate_enum_field(
            "override-snmp-user",
            payload["override-snmp-user"],
            VALID_BODY_OVERRIDE_SNMP_USER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "qos-drop-policy" in payload:
        is_valid, error = _validate_enum_field(
            "qos-drop-policy",
            payload["qos-drop-policy"],
            VALID_BODY_QOS_DROP_POLICY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "switch_controller/managed_switch",
    "category": "cmdb",
    "api_path": "switch-controller/managed-switch",
    "mkey": "switch-id",
    "mkey_type": "string",
    "help": "Configure FortiSwitch devices that are managed by this FortiGate.",
    "total_fields": 68,
    "required_fields_count": 3,
    "fields_with_defaults_count": 45,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""
Pydantic Models for CMDB - user/security_exempt_list

Runtime validation models for user/security_exempt_list configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class SecurityExemptListRuleSrcaddr(BaseModel):
    """
    Child table model for rule.srcaddr.
    
    Source addresses or address groups.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=79, default=None, description="Address or group name.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']
class SecurityExemptListRuleService(BaseModel):
    """
    Child table model for rule.service.
    
    Destination services.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=79, default=None, description="Service name.")  # datasource: ['firewall.service.custom.name', 'firewall.service.group.name']
class SecurityExemptListRuleDstaddr(BaseModel):
    """
    Child table model for rule.dstaddr.
    
    Destination addresses or address groups.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=79, default=None, description="Address or group name.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']
class SecurityExemptListRule(BaseModel):
    """
    Child table model for rule.
    
    Configure rules for exempting users from captive portal authentication.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    srcaddr: list[SecurityExemptListRuleSrcaddr] = Field(default_factory=list, description="Source addresses or address groups.")    
    dstaddr: list[SecurityExemptListRuleDstaddr] = Field(default_factory=list, description="Destination addresses or address groups.")    
    service: list[SecurityExemptListRuleService] = Field(default_factory=list, description="Destination services.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class SecurityExemptListModel(BaseModel):
    """
    Pydantic model for user/security_exempt_list configuration.
    
    Configure security exemption list.
    
    Validation Rules:        - name: max_length=35 pattern=        - description: max_length=127 pattern=        - rule: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="Name of the exempt list.")    
    description: str | None = Field(max_length=127, default=None, description="Description.")    
    rule: list[SecurityExemptListRule] = Field(default_factory=list, description="Configure rules for exempting users from captive portal authentication.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SecurityExemptListModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SecurityExemptListModel",    "SecurityExemptListRule",    "SecurityExemptListRule.Srcaddr",    "SecurityExemptListRule.Dstaddr",    "SecurityExemptListRule.Service",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.096196Z
# ============================================================================


"""
Pydantic Models for CMDB - system/admin

Runtime validation models for system/admin configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class AdminVdom(BaseModel):
    """
    Child table model for vdom.
    
    Virtual domain(s) that the administrator can access.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="Virtual domain name.")  # datasource: ['system.vdom.name']
class AdminGuestUsergroups(BaseModel):
    """
    Child table model for guest-usergroups.
    
    Select guest user groups.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="Select guest user groups.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class AdminTwoFactorEnum(str, Enum):
    """Allowed values for two_factor field."""
    DISABLE = "disable"
    FORTITOKEN = "fortitoken"
    FORTITOKEN_CLOUD = "fortitoken-cloud"
    EMAIL = "email"
    SMS = "sms"


# ============================================================================
# Main Model
# ============================================================================

class AdminModel(BaseModel):
    """
    Pydantic model for system/admin configuration.
    
    Configure admin users.
    
    Validation Rules:        - name: max_length=64 pattern=        - vdom: pattern=        - remote_auth: pattern=        - remote_group: max_length=35 pattern=        - wildcard: pattern=        - password: max_length=128 pattern=        - peer_auth: pattern=        - peer_group: max_length=35 pattern=        - trusthost1: pattern=        - trusthost2: pattern=        - trusthost3: pattern=        - trusthost4: pattern=        - trusthost5: pattern=        - trusthost6: pattern=        - trusthost7: pattern=        - trusthost8: pattern=        - trusthost9: pattern=        - trusthost10: pattern=        - ip6_trusthost1: pattern=        - ip6_trusthost2: pattern=        - ip6_trusthost3: pattern=        - ip6_trusthost4: pattern=        - ip6_trusthost5: pattern=        - ip6_trusthost6: pattern=        - ip6_trusthost7: pattern=        - ip6_trusthost8: pattern=        - ip6_trusthost9: pattern=        - ip6_trusthost10: pattern=        - accprofile: max_length=35 pattern=        - allow_remove_admin_session: pattern=        - comments: max_length=255 pattern=        - ssh_public_key1: pattern=        - ssh_public_key2: pattern=        - ssh_public_key3: pattern=        - ssh_certificate: max_length=35 pattern=        - schedule: max_length=35 pattern=        - accprofile_override: pattern=        - vdom_override: pattern=        - password_expire: pattern=        - force_password_change: pattern=        - two_factor: pattern=        - two_factor_authentication: pattern=        - two_factor_notification: pattern=        - fortitoken: max_length=16 pattern=        - email_to: max_length=63 pattern=        - sms_server: pattern=        - sms_custom_server: max_length=35 pattern=        - sms_phone: max_length=15 pattern=        - guest_auth: pattern=        - guest_usergroups: pattern=        - guest_lang: max_length=35 pattern=        - status: pattern=        - list_: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=64, default=None, description="User name.")    
    vdom: list[AdminVdom] = Field(default_factory=list, description="Virtual domain(s) that the administrator can access.")    
    remote_auth: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable authentication using a remote RADIUS, LDAP, or TACACS+ server.")    
    remote_group: str = Field(max_length=35, description="User group name used for remote auth.")    
    wildcard: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable wildcard RADIUS authentication.")    
    password: Any = Field(max_length=128, description="Admin user password.")    
    peer_auth: Literal["enable", "disable"] | None = Field(default="disable", description="Set to enable peer certificate authentication (for HTTPS admin access).")    
    peer_group: str = Field(max_length=35, description="Name of peer group defined under config user group which has PKI members. Used for peer certificate authentication (for HTTPS admin access).")    
    trusthost1: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    trusthost2: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    trusthost3: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    trusthost4: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    trusthost5: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    trusthost6: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    trusthost7: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    trusthost8: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    trusthost9: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    trusthost10: str | None = Field(default="0.0.0.0 0.0.0.0", description="Any IPv4 address or subnet address and netmask from which the administrator can connect to the FortiGate unit. Default allows access from any IPv4 address.")    
    ip6_trusthost1: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    ip6_trusthost2: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    ip6_trusthost3: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    ip6_trusthost4: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    ip6_trusthost5: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    ip6_trusthost6: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    ip6_trusthost7: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    ip6_trusthost8: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    ip6_trusthost9: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    ip6_trusthost10: str | None = Field(default="::/0", description="Any IPv6 address from which the administrator can connect to the FortiGate unit. Default allows access from any IPv6 address.")    
    accprofile: str | None = Field(max_length=35, default=None, description="Access profile for this administrator. Access profiles control administrator access to FortiGate features.")  # datasource: ['system.accprofile.name']    
    allow_remove_admin_session: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable allow admin session to be removed by privileged admin users.")    
    comments: str | None = Field(max_length=255, default=None, description="Comment.")    
    ssh_public_key1: str | None = Field(default=None, description="Public key of an SSH client. The client is authenticated without being asked for credentials. Create the public-private key pair in the SSH client application.")    
    ssh_public_key2: str | None = Field(default=None, description="Public key of an SSH client. The client is authenticated without being asked for credentials. Create the public-private key pair in the SSH client application.")    
    ssh_public_key3: str | None = Field(default=None, description="Public key of an SSH client. The client is authenticated without being asked for credentials. Create the public-private key pair in the SSH client application.")    
    ssh_certificate: str | None = Field(max_length=35, default=None, description="Select the certificate to be used by the FortiGate for authentication with an SSH client.")  # datasource: ['certificate.remote.name']    
    schedule: str | None = Field(max_length=35, default=None, description="Firewall schedule used to restrict when the administrator can log in. No schedule means no restrictions.")    
    accprofile_override: Literal["enable", "disable"] | None = Field(default="disable", description="Enable to use the name of an access profile provided by the remote authentication server to control the FortiGate features that this administrator can access.")    
    vdom_override: Literal["enable", "disable"] | None = Field(default="disable", description="Enable to use the names of VDOMs provided by the remote authentication server to control the VDOMs that this administrator can access.")    
    password_expire: Any = Field(default="0000-00-00 00:00:00", description="Password expire time.")    
    force_password_change: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable force password change on next login.")    
    two_factor: AdminTwoFactorEnum | None = Field(default=AdminTwoFactorEnum.DISABLE, description="Enable/disable two-factor authentication.")    
    two_factor_authentication: Literal["fortitoken", "email", "sms"] | None = Field(default=None, description="Authentication method by FortiToken Cloud.")    
    two_factor_notification: Literal["email", "sms"] | None = Field(default=None, description="Notification method for user activation by FortiToken Cloud.")    
    fortitoken: str | None = Field(max_length=16, default=None, description="This administrator's FortiToken serial number.")    
    email_to: str | None = Field(max_length=63, default=None, description="This administrator's email address.")    
    sms_server: Literal["fortiguard", "custom"] | None = Field(default="fortiguard", description="Send SMS messages using the FortiGuard SMS server or a custom server.")    
    sms_custom_server: str | None = Field(max_length=35, default=None, description="Custom SMS server to send SMS messages to.")  # datasource: ['system.sms-server.name']    
    sms_phone: str | None = Field(max_length=15, default=None, description="Phone number on which the administrator receives SMS messages.")    
    guest_auth: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable guest authentication.")    
    guest_usergroups: list[AdminGuestUsergroups] = Field(default_factory=list, description="Select guest user groups.")    
    guest_lang: str | None = Field(max_length=35, default=None, description="Guest management portal language.")  # datasource: ['system.custom-language.name']    
    status: Any = Field(default=None, description="print admin status information")    
    list_: Any = Field(default=None, serialization_alias="list", description="print admin list information")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('accprofile')
    @classmethod
    def validate_accprofile(cls, v: Any) -> Any:
        """
        Validate accprofile field.
        
        Datasource: ['system.accprofile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('ssh_certificate')
    @classmethod
    def validate_ssh_certificate(cls, v: Any) -> Any:
        """
        Validate ssh_certificate field.
        
        Datasource: ['certificate.remote.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('sms_custom_server')
    @classmethod
    def validate_sms_custom_server(cls, v: Any) -> Any:
        """
        Validate sms_custom_server field.
        
        Datasource: ['system.sms-server.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('guest_lang')
    @classmethod
    def validate_guest_lang(cls, v: Any) -> Any:
        """
        Validate guest_lang field.
        
        Datasource: ['system.custom-language.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "AdminModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_vdom_references(self, client: Any) -> list[str]:
        """
        Validate vdom references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/vdom        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AdminModel(
            ...     vdom=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_vdom_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.admin.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "vdom", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.vdom.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Vdom '{value}' not found in "
                    "system/vdom"
                )        
        return errors    
    async def validate_accprofile_references(self, client: Any) -> list[str]:
        """
        Validate accprofile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/accprofile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AdminModel(
            ...     accprofile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_accprofile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.admin.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "accprofile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.accprofile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Accprofile '{value}' not found in "
                "system/accprofile"
            )        
        return errors    
    async def validate_ssh_certificate_references(self, client: Any) -> list[str]:
        """
        Validate ssh_certificate references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - certificate/remote        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AdminModel(
            ...     ssh_certificate="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ssh_certificate_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.admin.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ssh_certificate", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.certificate.remote.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ssh-Certificate '{value}' not found in "
                "certificate/remote"
            )        
        return errors    
    async def validate_sms_custom_server_references(self, client: Any) -> list[str]:
        """
        Validate sms_custom_server references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/sms-server        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AdminModel(
            ...     sms_custom_server="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_sms_custom_server_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.admin.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "sms_custom_server", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.sms_server.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Sms-Custom-Server '{value}' not found in "
                "system/sms-server"
            )        
        return errors    
    async def validate_guest_lang_references(self, client: Any) -> list[str]:
        """
        Validate guest_lang references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/custom-language        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AdminModel(
            ...     guest_lang="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_guest_lang_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.admin.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "guest_lang", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.custom_language.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Guest-Lang '{value}' not found in "
                "system/custom-language"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_vdom_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_accprofile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ssh_certificate_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_sms_custom_server_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_guest_lang_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "AdminModel",    "AdminVdom",    "AdminGuestUsergroups",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.416368Z
# ============================================================================


"""Validation helpers for system/link_monitor - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "server",  # IP address of the server(s) to be monitored.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "addr-mode": "ipv4",
    "srcintf": "",
    "server-config": "default",
    "server-type": "static",
    "protocol": "ping",
    "port": 0,
    "gateway-ip": "0.0.0.0",
    "gateway-ip6": "::",
    "source-ip": "0.0.0.0",
    "source-ip6": "::",
    "http-get": "/",
    "http-agent": "Chrome/ Safari/",
    "http-match": "",
    "interval": 500,
    "probe-timeout": 500,
    "failtime": 5,
    "recoverytime": 5,
    "probe-count": 30,
    "security-mode": "none",
    "packet-size": 124,
    "ha-priority": 1,
    "fail-weight": 0,
    "update-cascade-interface": "enable",
    "update-static-route": "enable",
    "update-policy-route": "enable",
    "status": "enable",
    "diffservcode": "",
    "class-id": 0,
    "service-detection": "disable",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # Link monitor name.
    "addr-mode": "option",  # Address mode (IPv4 or IPv6).
    "srcintf": "string",  # Interface that receives the traffic to be monitored.
    "server-config": "option",  # Mode of server configuration.
    "server-type": "option",  # Server type (static or dynamic).
    "server": "string",  # IP address of the server(s) to be monitored.
    "protocol": "option",  # Protocols used to monitor the server.
    "port": "integer",  # Port number of the traffic to be used to monitor the server.
    "gateway-ip": "ipv4-address-any",  # Gateway IP address used to probe the server.
    "gateway-ip6": "ipv6-address",  # Gateway IPv6 address used to probe the server.
    "route": "string",  # Subnet to monitor.
    "source-ip": "ipv4-address-any",  # Source IP address used in packet to the server.
    "source-ip6": "ipv6-address",  # Source IPv6 address used in packet to the server.
    "http-get": "string",  # If you are monitoring an HTML server you can send an HTTP-GE
    "http-agent": "string",  # String in the http-agent field in the HTTP header.
    "http-match": "string",  # String that you expect to see in the HTTP-GET requests of th
    "interval": "integer",  # Detection interval in milliseconds (20 - 3600 * 1000 msec, d
    "probe-timeout": "integer",  # Time to wait before a probe packet is considered lost (20 - 
    "failtime": "integer",  # Number of retry attempts before the server is considered dow
    "recoverytime": "integer",  # Number of successful responses received before server is con
    "probe-count": "integer",  # Number of most recent probes that should be used to calculat
    "security-mode": "option",  # Twamp controller security mode.
    "password": "password",  # TWAMP controller password in authentication mode.
    "packet-size": "integer",  # Packet size of a TWAMP test session (124/158 - 1024).
    "ha-priority": "integer",  # HA election priority (1 - 50).
    "fail-weight": "integer",  # Threshold weight to trigger link failure alert.
    "update-cascade-interface": "option",  # Enable/disable update cascade interface.
    "update-static-route": "option",  # Enable/disable updating the static route.
    "update-policy-route": "option",  # Enable/disable updating the policy route.
    "status": "option",  # Enable/disable this link monitor.
    "diffservcode": "user",  # Differentiated services code point (DSCP) in the IP header o
    "class-id": "integer",  # Traffic class ID.
    "service-detection": "option",  # Only use monitor to read quality values. If enabled, static 
    "server-list": "string",  # Servers for link-monitor to monitor.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "Link monitor name.",
    "addr-mode": "Address mode (IPv4 or IPv6).",
    "srcintf": "Interface that receives the traffic to be monitored.",
    "server-config": "Mode of server configuration.",
    "server-type": "Server type (static or dynamic).",
    "server": "IP address of the server(s) to be monitored.",
    "protocol": "Protocols used to monitor the server.",
    "port": "Port number of the traffic to be used to monitor the server.",
    "gateway-ip": "Gateway IP address used to probe the server.",
    "gateway-ip6": "Gateway IPv6 address used to probe the server.",
    "route": "Subnet to monitor.",
    "source-ip": "Source IP address used in packet to the server.",
    "source-ip6": "Source IPv6 address used in packet to the server.",
    "http-get": "If you are monitoring an HTML server you can send an HTTP-GET request with a custom string. Use this option to define the string.",
    "http-agent": "String in the http-agent field in the HTTP header.",
    "http-match": "String that you expect to see in the HTTP-GET requests of the traffic to be monitored.",
    "interval": "Detection interval in milliseconds (20 - 3600 * 1000 msec, default = 500).",
    "probe-timeout": "Time to wait before a probe packet is considered lost (20 - 5000 msec, default = 500).",
    "failtime": "Number of retry attempts before the server is considered down (1 - 3600, default = 5).",
    "recoverytime": "Number of successful responses received before server is considered recovered (1 - 3600, default = 5).",
    "probe-count": "Number of most recent probes that should be used to calculate latency and jitter (5 - 30, default = 30).",
    "security-mode": "Twamp controller security mode.",
    "password": "TWAMP controller password in authentication mode.",
    "packet-size": "Packet size of a TWAMP test session (124/158 - 1024).",
    "ha-priority": "HA election priority (1 - 50).",
    "fail-weight": "Threshold weight to trigger link failure alert.",
    "update-cascade-interface": "Enable/disable update cascade interface.",
    "update-static-route": "Enable/disable updating the static route.",
    "update-policy-route": "Enable/disable updating the policy route.",
    "status": "Enable/disable this link monitor.",
    "diffservcode": "Differentiated services code point (DSCP) in the IP header of the probe packet.",
    "class-id": "Traffic class ID.",
    "service-detection": "Only use monitor to read quality values. If enabled, static routes and cascade interfaces will not be updated.",
    "server-list": "Servers for link-monitor to monitor.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 35},
    "srcintf": {"type": "string", "max_length": 15},
    "port": {"type": "integer", "min": 1, "max": 65535},
    "http-get": {"type": "string", "max_length": 1024},
    "http-agent": {"type": "string", "max_length": 1024},
    "http-match": {"type": "string", "max_length": 1024},
    "interval": {"type": "integer", "min": 20, "max": 3600000},
    "probe-timeout": {"type": "integer", "min": 20, "max": 5000},
    "failtime": {"type": "integer", "min": 1, "max": 3600},
    "recoverytime": {"type": "integer", "min": 1, "max": 3600},
    "probe-count": {"type": "integer", "min": 5, "max": 30},
    "packet-size": {"type": "integer", "min": 0, "max": 65535},
    "ha-priority": {"type": "integer", "min": 1, "max": 50},
    "fail-weight": {"type": "integer", "min": 0, "max": 255},
    "class-id": {"type": "integer", "min": 0, "max": 4294967295},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "server": {
        "address": {
            "type": "string",
            "help": "Server address.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
    },
    "route": {
        "subnet": {
            "type": "string",
            "help": "IP and netmask (x.x.x.x/y).",
            "default": "",
            "max_length": 79,
        },
    },
    "server-list": {
        "id": {
            "type": "integer",
            "help": "Server ID.",
            "required": True,
            "default": 0,
            "min_value": 1,
            "max_value": 32,
        },
        "dst": {
            "type": "string",
            "help": "IP address of the server to be monitored.",
            "required": True,
            "default": "",
            "max_length": 64,
        },
        "protocol": {
            "type": "option",
            "help": "Protocols used to monitor the server.",
            "default": "ping",
            "options": ["ping", "tcp-echo", "udp-echo", "http", "https", "twamp"],
        },
        "port": {
            "type": "integer",
            "help": "Port number of the traffic to be used to monitor the server.",
            "default": 0,
            "min_value": 1,
            "max_value": 65535,
        },
        "weight": {
            "type": "integer",
            "help": "Weight of the monitor to this dst (0 - 255).",
            "default": 0,
            "min_value": 0,
            "max_value": 255,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_ADDR_MODE = [
    "ipv4",
    "ipv6",
]
VALID_BODY_SERVER_CONFIG = [
    "default",
    "individual",
]
VALID_BODY_SERVER_TYPE = [
    "static",
    "dynamic",
]
VALID_BODY_PROTOCOL = [
    "ping",
    "tcp-echo",
    "udp-echo",
    "http",
    "https",
    "twamp",
]
VALID_BODY_SECURITY_MODE = [
    "none",
    "authentication",
]
VALID_BODY_UPDATE_CASCADE_INTERFACE = [
    "enable",
    "disable",
]
VALID_BODY_UPDATE_STATIC_ROUTE = [
    "enable",
    "disable",
]
VALID_BODY_UPDATE_POLICY_ROUTE = [
    "enable",
    "disable",
]
VALID_BODY_STATUS = [
    "enable",
    "disable",
]
VALID_BODY_SERVICE_DETECTION = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_system_link_monitor_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for system/link_monitor."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_system_link_monitor_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new system/link_monitor object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "addr-mode" in payload:
        is_valid, error = _validate_enum_field(
            "addr-mode",
            payload["addr-mode"],
            VALID_BODY_ADDR_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-config" in payload:
        is_valid, error = _validate_enum_field(
            "server-config",
            payload["server-config"],
            VALID_BODY_SERVER_CONFIG,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-type" in payload:
        is_valid, error = _validate_enum_field(
            "server-type",
            payload["server-type"],
            VALID_BODY_SERVER_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "protocol" in payload:
        is_valid, error = _validate_enum_field(
            "protocol",
            payload["protocol"],
            VALID_BODY_PROTOCOL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "security-mode" in payload:
        is_valid, error = _validate_enum_field(
            "security-mode",
            payload["security-mode"],
            VALID_BODY_SECURITY_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "update-cascade-interface" in payload:
        is_valid, error = _validate_enum_field(
            "update-cascade-interface",
            payload["update-cascade-interface"],
            VALID_BODY_UPDATE_CASCADE_INTERFACE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "update-static-route" in payload:
        is_valid, error = _validate_enum_field(
            "update-static-route",
            payload["update-static-route"],
            VALID_BODY_UPDATE_STATIC_ROUTE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "update-policy-route" in payload:
        is_valid, error = _validate_enum_field(
            "update-policy-route",
            payload["update-policy-route"],
            VALID_BODY_UPDATE_POLICY_ROUTE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "status" in payload:
        is_valid, error = _validate_enum_field(
            "status",
            payload["status"],
            VALID_BODY_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "service-detection" in payload:
        is_valid, error = _validate_enum_field(
            "service-detection",
            payload["service-detection"],
            VALID_BODY_SERVICE_DETECTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_system_link_monitor_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update system/link_monitor."""
    # Validate enum values using central function
    if "addr-mode" in payload:
        is_valid, error = _validate_enum_field(
            "addr-mode",
            payload["addr-mode"],
            VALID_BODY_ADDR_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-config" in payload:
        is_valid, error = _validate_enum_field(
            "server-config",
            payload["server-config"],
            VALID_BODY_SERVER_CONFIG,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-type" in payload:
        is_valid, error = _validate_enum_field(
            "server-type",
            payload["server-type"],
            VALID_BODY_SERVER_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "protocol" in payload:
        is_valid, error = _validate_enum_field(
            "protocol",
            payload["protocol"],
            VALID_BODY_PROTOCOL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "security-mode" in payload:
        is_valid, error = _validate_enum_field(
            "security-mode",
            payload["security-mode"],
            VALID_BODY_SECURITY_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "update-cascade-interface" in payload:
        is_valid, error = _validate_enum_field(
            "update-cascade-interface",
            payload["update-cascade-interface"],
            VALID_BODY_UPDATE_CASCADE_INTERFACE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "update-static-route" in payload:
        is_valid, error = _validate_enum_field(
            "update-static-route",
            payload["update-static-route"],
            VALID_BODY_UPDATE_STATIC_ROUTE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "update-policy-route" in payload:
        is_valid, error = _validate_enum_field(
            "update-policy-route",
            payload["update-policy-route"],
            VALID_BODY_UPDATE_POLICY_ROUTE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "status" in payload:
        is_valid, error = _validate_enum_field(
            "status",
            payload["status"],
            VALID_BODY_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "service-detection" in payload:
        is_valid, error = _validate_enum_field(
            "service-detection",
            payload["service-detection"],
            VALID_BODY_SERVICE_DETECTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "system/link_monitor",
    "category": "cmdb",
    "api_path": "system/link-monitor",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Configure Link Health Monitor.",
    "total_fields": 34,
    "required_fields_count": 1,
    "fields_with_defaults_count": 30,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""
Pydantic Models for CMDB - system/ntp

Runtime validation models for system/ntp configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class NtpNtpserver(BaseModel):
    """
    Child table model for ntpserver.
    
    Configure the FortiGate to connect to any available third-party NTP server.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="NTP server ID.")    
    server: str = Field(max_length=63, description="IP address or hostname of the NTP Server.")    
    ntpv3: Literal["enable", "disable"] | None = Field(default="disable", description="Enable to use NTPv3 instead of NTPv4.")    
    authentication: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable authentication.")    
    key_type: Literal["MD5", "SHA1", "SHA256"] | None = Field(default="MD5", description="Select NTP authentication type.")    
    key: Any = Field(max_length=64, description="Key for MD5(NTPv3)/SHA1(NTPv4)/SHA256(NTPv4) authentication.")    
    key_id: int = Field(ge=0, le=4294967295, default=0, description="Key ID for authentication.")    
    ip_type: Literal["IPv6", "IPv4", "Both"] | None = Field(default="Both", description="Choose to connect to IPv4 or/and IPv6 NTP server.")    
    interface_select_method: Literal["auto", "sdwan", "specify"] | None = Field(default="auto", description="Specify how to select outgoing interface to reach server.")    
    interface: str = Field(max_length=15, description="Specify outgoing interface to reach server.")  # datasource: ['system.interface.name']    
    vrf_select: int | None = Field(ge=0, le=511, default=0, description="VRF ID used for connection to server.")
class NtpInterface(BaseModel):
    """
    Child table model for interface.
    
    FortiGate interface(s) with NTP server mode enabled. Devices on your network can contact these interfaces for NTP services.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    interface_name: str = Field(max_length=79, description="Interface name.")  # datasource: ['system.interface.name']
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class NtpModel(BaseModel):
    """
    Pydantic model for system/ntp configuration.
    
    Configure system NTP information.
    
    Validation Rules:        - ntpsync: pattern=        - type_: pattern=        - syncinterval: min=1 max=1440 pattern=        - ntpserver: pattern=        - source_ip: pattern=        - source_ip6: pattern=        - server_mode: pattern=        - authentication: pattern=        - key_type: pattern=        - key: max_length=64 pattern=        - key_id: min=0 max=4294967295 pattern=        - interface: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    ntpsync: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable setting the FortiGate system time by synchronizing with an NTP Server.")    
    type_: Literal["fortiguard", "custom"] | None = Field(default="fortiguard", serialization_alias="type", description="Use the FortiGuard NTP server or any other available NTP Server.")    
    syncinterval: int | None = Field(ge=1, le=1440, default=60, description="NTP synchronization interval (1 - 1440 min).")    
    ntpserver: list[NtpNtpserver] = Field(default_factory=list, description="Configure the FortiGate to connect to any available third-party NTP server.")    
    source_ip: str | None = Field(default="0.0.0.0", description="Source IP address for communication to the NTP server.")    
    source_ip6: str | None = Field(default="::", description="Source IPv6 address for communication to the NTP server.")    
    server_mode: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable FortiGate NTP Server Mode. Your FortiGate becomes an NTP server for other devices on your network. The FortiGate relays NTP requests to its configured NTP server.")    
    authentication: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable authentication.")    
    key_type: Literal["MD5", "SHA1", "SHA256"] | None = Field(default="MD5", description="Key type for authentication (MD5, SHA1, SHA256).")    
    key: Any = Field(max_length=64, description="Key for authentication.")    
    key_id: int = Field(ge=0, le=4294967295, default=0, description="Key ID for authentication.")    
    interface: list[NtpInterface] = Field(default_factory=list, description="FortiGate interface(s) with NTP server mode enabled. Devices on your network can contact these interfaces for NTP services.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "NtpModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_ntpserver_references(self, client: Any) -> list[str]:
        """
        Validate ntpserver references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = NtpModel(
            ...     ntpserver=[{"interface": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ntpserver_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.ntp.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "ntpserver", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("interface")
            else:
                value = getattr(item, "interface", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.interface.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Ntpserver '{value}' not found in "
                    "system/interface"
                )        
        return errors    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = NtpModel(
            ...     interface=[{"interface-name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.ntp.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "interface", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("interface-name")
            else:
                value = getattr(item, "interface-name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.interface.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Interface '{value}' not found in "
                    "system/interface"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_ntpserver_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "NtpModel",    "NtpNtpserver",    "NtpInterface",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.325637Z
# ============================================================================


from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_STATUS: Literal["enable", "disable"]
VALID_BODY_SECURE_WEB_PROXY: Literal["disable", "enable", "secure"]
VALID_BODY_FTP_OVER_HTTP: Literal["enable", "disable"]
VALID_BODY_SOCKS: Literal["enable", "disable"]
VALID_BODY_HTTP_CONNECTION_MODE: Literal["static", "multiplex", "serverpool"]
VALID_BODY_CLIENT_CERT: Literal["disable", "enable"]
VALID_BODY_USER_AGENT_DETECT: Literal["disable", "enable"]
VALID_BODY_EMPTY_CERT_ACTION: Literal["accept", "block", "accept-unmanageable"]
VALID_BODY_SSL_DH_BITS: Literal["768", "1024", "1536", "2048"]
VALID_BODY_INTERFACE_SELECT_METHOD: Literal["sdwan", "specify"]
VALID_BODY_IPV6_STATUS: Literal["enable", "disable"]
VALID_BODY_STRICT_GUEST: Literal["enable", "disable"]
VALID_BODY_PREF_DNS_RESULT: Literal["ipv4", "ipv6", "ipv4-strict", "ipv6-strict"]
VALID_BODY_UNKNOWN_HTTP_VERSION: Literal["reject", "best-effort"]
VALID_BODY_SEC_DEFAULT_ACTION: Literal["accept", "deny"]
VALID_BODY_HTTPS_REPLACEMENT_MESSAGE: Literal["enable", "disable"]
VALID_BODY_MESSAGE_UPON_SERVER_ERROR: Literal["enable", "disable"]
VALID_BODY_PAC_FILE_SERVER_STATUS: Literal["enable", "disable"]
VALID_BODY_PAC_FILE_THROUGH_HTTPS: Literal["enable", "disable"]
VALID_BODY_SSL_ALGORITHM: Literal["high", "medium", "low"]
VALID_BODY_TRACE_AUTH_NO_RSP: Literal["enable", "disable"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_STATUS",
    "VALID_BODY_SECURE_WEB_PROXY",
    "VALID_BODY_FTP_OVER_HTTP",
    "VALID_BODY_SOCKS",
    "VALID_BODY_HTTP_CONNECTION_MODE",
    "VALID_BODY_CLIENT_CERT",
    "VALID_BODY_USER_AGENT_DETECT",
    "VALID_BODY_EMPTY_CERT_ACTION",
    "VALID_BODY_SSL_DH_BITS",
    "VALID_BODY_INTERFACE_SELECT_METHOD",
    "VALID_BODY_IPV6_STATUS",
    "VALID_BODY_STRICT_GUEST",
    "VALID_BODY_PREF_DNS_RESULT",
    "VALID_BODY_UNKNOWN_HTTP_VERSION",
    "VALID_BODY_SEC_DEFAULT_ACTION",
    "VALID_BODY_HTTPS_REPLACEMENT_MESSAGE",
    "VALID_BODY_MESSAGE_UPON_SERVER_ERROR",
    "VALID_BODY_PAC_FILE_SERVER_STATUS",
    "VALID_BODY_PAC_FILE_THROUGH_HTTPS",
    "VALID_BODY_SSL_ALGORITHM",
    "VALID_BODY_TRACE_AUTH_NO_RSP",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""Type stubs for SECURITY_RATING category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .recommendations import Recommendations
    from .report import Report

__all__ = [
    "Recommendations",
    "Report",
    "SecurityRating",
]


class SecurityRating:
    """SECURITY_RATING API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    recommendations: Recommendations
    report: Report

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize security_rating category with HTTP client."""
        ...



"""FortiOS CMDB - PasswordPolicyConform category"""

from .select import Select

__all__ = [
    "PasswordPolicyConform",
    "Select",
]


class PasswordPolicyConform:
    """PasswordPolicyConform endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """PasswordPolicyConform endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.select = Select(client)
